% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MR.quantile.r
\name{MR.quantile}
\alias{MR.quantile}
\title{Multiply Robust Estimation of the Marginal Quantile}
\usage{
MR.quantile(response, tau = 0.5, imp.model = NULL, mis.model = NULL,
  L = 30, data, bootstrap = FALSE, bootstrap.size = 500,
  alpha = 0.05)
}
\arguments{
\item{response}{The response variable of interest whose marginal quantile is to be estimated.}

\item{tau}{A numeric value in (0,1). The quantile to be estimated.}

\item{imp.model}{A list of imputation models defined by \code{\link{def.glm}}.}

\item{mis.model}{A list of missingness probability models defined by \code{\link{def.glm}}. The dependent variable is always specified as \code{R}.}

\item{L}{Number of imputations.}

\item{data}{A data frame with missing data encoded as \code{NA}.}

\item{bootstrap}{Logical. Should a bootstrap method be applied to calculate the standard error of the estimator and construct a Wald confidence interval for the estimated marginal quantile.}

\item{bootstrap.size}{A numeric value. Number of bootstrap resamples generated if \code{bootstrap = TRUE}.}

\item{alpha}{Significance level used to construct the 100(1 - \code{alpha})\% Wald confidence interval.}
}
\value{
\item{\code{q}}{The estimated value of the marginal quantile.}
\item{\code{SE}}{The bootstrap standard error of \code{q} when \code{bootstrap = TRUE}.}
\item{\code{CI}}{A Wald-type confidence interval based on \code{q} and \code{SE} when \code{bootstrap = TRUE}.}
}
\description{
\code{MR.quantile()} is used to estimate the marginal quantile of a variable which is subject to missingness. Multiple missingness probability models and imputation models are allowed.
}
\examples{
# Simulated data set
set.seed(123)
n <- 400
gamma0 <- c(1, 2, 3)
alpha0 <- c(-0.8, -0.5, 0.3)
X <- runif(n, min = -2.5, max = 2.5)
p.mis <- 1 / (1 + exp(alpha0[1] + alpha0[2] * X + alpha0[3] * X ^ 2))
R <- rbinom(n, size = 1, prob = 1 - p.mis)
a.x <- gamma0[1] + gamma0[2] * X + gamma0[3] * exp(X)
Y <- rnorm(n, a.x, sd = sqrt(4 * X ^ 2 + 2))
dat <- data.frame(X, Y)
dat[R == 0, 2] <- NA

# Define the outcome regression models and missingness probability models
imp1 <- def.glm(formula = Y ~ X + exp(X), family = gaussian)
imp2 <- def.glm(formula = Y ~ X + X ^ 2, family = gaussian)
mis1 <- def.glm(formula = R ~ X + X ^ 2, family = binomial(link = logit))
mis2 <- def.glm(formula = R ~ X + exp(X), family = binomial(link = cloglog))
est <- MR.quantile(response = Y, tau = 0.25, imp.model = list(imp1, imp2),
                   mis.model = list(mis1, mis2), L = 10, data = dat)
est$q
}
\references{
Han, P., Kong, L., Zhao, J. and Zhou, X. (2019). A general framework for quantile estimation with incomplete data. \emph{Journal of the Royal Statistical Society: Series B (Statistical Methodology)}. In press.
}
